const puppeteer = require('puppeteer-core');
const fs = require('fs-extra');
const path = require('path');
const axios = require('axios');

// Cấu hình dễ thay đổi
const baseFolder = 'D:/NickTeleChrome';
const telegramUrl = 'https://web.telegram.org/k/#?tgaddr=tg%3A%2F%2Fresolve%3Fdomain%3Dnotpixel%26appname%3Dapp%26startapp%3Df6929484517_s662873';  // Đường dẫn đến Telegram
const delayBetweenRequests = 5000; // số giây giữa các request
const delayBetweenFolder   = 10000; // số giây giữa các request
const delayBetweenCycles = 15 * 60 * 1000; // 15 phút (đơn vị là milliseconds)
const headlessBrowser = false;  // true để chạy headless (ẩn trình duyệt), false để hiện trình duyệt
const disableInfo = false;  // true để tắt log thông tin ra console, false để hiện thông tin log

// Giá trị mặc định nếu API lỗi
const defaultPixelId = 783836;
const defaultNewColor = '#BE0039';

// API endpoint để lấy danh sách pixelId và newColor
const apiEndpoint = 'https://cdnstoremedia.com/adt/amt/2022/thangtq/not_pixel.json';  // API giả định, bạn cần thay đổi đường dẫn chính xác

// Giá trị nâng cấp các boosts
const boostCosts = {
    2: 5,
    3: 200,
    4: 300,
    5: 400,
    6: 500,
    7: 600,
    8: 700,
    9: 800,
};

// Hàm hỗ trợ ghi log theo cấu hình disableInfo
function logInfo(message, type = 'info') {
    const typeMap = {
        info: '[INFO]',
        success: '[SUCCESS]',
        error: '[ERROR]',
        warning: '[WARNING]'
    };
    console.log(`${typeMap[type]} ${message}`);
}

// Hàm delay (chờ giữa các request)
function delay(ms) {
    return new Promise(resolve => setTimeout(resolve, ms));
}

// Hàm lấy danh sách các folder con trong baseFolder
async function getSubfolders(baseFolder) {
    try {
        const folders = await fs.readdir(baseFolder);
        // Kiểm tra từng folder xem có phải là thư mục không
        const subfolders = [];
        for (const folder of folders) {
            const folderPath = path.join(baseFolder, folder);
            const stat = await fs.lstat(folderPath);
            if (stat.isDirectory()) {
                subfolders.push(folderPath);
            }
        }
        return subfolders;
    } catch (error) {
        logInfo(`Lỗi khi lấy danh sách folder: ${error.message}`, 'error');
        return [];
    }
}

// Hàm mở Chrome với profile tương ứng
async function openChromeWithProfile(chromePath, userDataDir) {
    const browser = await puppeteer.launch({
        executablePath: chromePath,
        userDataDir,
        headless: headlessBrowser,  // Sử dụng biến headlessBrowser
        args: ['--no-sandbox', '--disable-setuid-sandbox']
    });
    return browser;
}

// Hàm điều khiển Telegram Web và lấy tgWebAppData
async function controlTelegram(browser) {
    const page = await browser.newPage();
    await page.goto(telegramUrl, { waitUntil: 'networkidle0' });

    // Ấn vào nút 'Launch'
    await page.waitForSelector('button.popup-button.btn.primary.rp', { visible: true });
    await page.click('button.popup-button.btn.primary.rp');
    await delay(3000);

    // Chờ iframe xuất hiện
    await page.waitForSelector('iframe');
    const iframe = await page.$('iframe');
    const iframeSrc = await iframe.evaluate(el => el.src);

    // Tách giá trị tgWebAppData từ iframe
    const tgWebAppDataMatch = iframeSrc.match(/tgWebAppData=([^&]+)/);
    if (!tgWebAppDataMatch) {
        throw new Error('Không tìm thấy tgWebAppData');
    }
    const tgWebAppData = decodeURIComponent(tgWebAppDataMatch[1]);
    return tgWebAppData;
}

async function getPixelDataFromAPI() {
    try {
        const response = await axios.get(apiEndpoint);
        const pixels = response.data;

        if (pixels && pixels.length > 0) {
            return pixels;  // Trả về danh sách các cặp pixelId và newColor (nếu API trả về danh sách)
        } else {
            throw new Error('API trả về dữ liệu trống, dùng giá trị mặc định');
        }
    } catch (error) {
        logInfo(`Lỗi khi gọi API lấy pixelId và newColor, dùng giá trị mặc định: ${error.message}`, 'warning');
        return [{ pixelId: defaultPixelId, newColor: defaultNewColor }];  // Trả về mảng với cặp giá trị mặc định nếu API lỗi
    }
}

async function claimReward(tgWebAppData) {
    const session = axios.create({
        headers: {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/129.0.0.0 Safari/537.36 Edg/129.0.0.0',
            'Authorization': `initData ${tgWebAppData}`,
            'authority': 'notpx.app',
            'method': 'GET',
            'path': '/api/v1/mining/claim',
            'scheme': 'https',
            'accept': 'application/json, text/plain, */*',
            'accept-encoding': 'gzip, deflate, br, zstd',
            'accept-language': 'en-US,en;q=0.9',
            'cache-control': 'no-cache',
            'origin': 'https://app.notpx.app',
            'pragma': 'no-cache',
            'priority': 'u=1, i',
            'referer': 'https://app.notpx.app/',
            'sec-ch-ua': '"Microsoft Edge";v="129", "Not=A?Brand";v="8", "Chromium";v="129", "Microsoft Edge WebView2";v="129"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"Windows"',
            'sec-fetch-dest': 'empty',
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'same-site',
        }
    });

    let attempt = 0;
    let maxAttempts = 3; // Tổng cộng 3 lần thử (lần đầu tiên + 2 lần thử lại)
    let success = false;

    while (attempt < maxAttempts && !success) {
        try {
            const response = await session.get('https://notpx.app/api/v1/mining/claim');
            logInfo('Claim reward thành công!', 'success');
            logInfo(`Chi tiết response từ API claim: ${JSON.stringify(response.data)}`, 'info');
            
            success = true; // Request thành công, dừng việc thử lại
        } catch (error) {
            const statusCode = error.response ? error.response.status : null;
            attempt++;
            if (statusCode === 400 || statusCode === 504) {
                logInfo(`Thử lại lần ${attempt}/${maxAttempts} cho request claim do lỗi ${statusCode}`, 'warning');
                await delay(2000); // Chờ 2 giây trước khi thử lại
            } else {
                logInfo(`Lỗi khi gửi request claim: ${error.response ? error.response.status : error.message}`, 'error');
                break; // Nếu không phải lỗi 400 hoặc 504 thì thoát vòng lặp
            }
        }
    }

    if (!success) {
        logInfo(`Request claim thất bại sau ${maxAttempts} lần thử.`, 'error');
    }
}



// Hàm xử lý toàn bộ quá trình mining, tasks, boosts, và gửi request repaint/start
async function handleMiningStatus(tgWebAppData) {
    const session = axios.create({
        headers: {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/129.0.0.0 Safari/537.36',
            'Authorization': `initData ${tgWebAppData}`,
            'Origin': 'https://app.notpx.app',
            'Referer': 'https://app.notpx.app/',
            'accept': 'application/json, text/plain, */*',
            'cache-control': 'no-cache',
        }
    });

    let attempts = 0;
    const maxAttempts = 3;
    const retryDelay = 2000; // 2 giây giữa các lần thử

    while (attempts < maxAttempts) {
        try {
            const response = await session.get('https://notpx.app/api/v1/mining/status');
            logInfo('Response từ mining/status:', 'info');
            logInfo(`Chi tiết response từ API mining/status: ${JSON.stringify(response.data)}`, 'info');
            const { userBalance, charges, league, tasks, boosts } = response.data;
            
            // Ghi lại các thông số
            logInfo(`userBalance: ${userBalance}`);
            logInfo(`charges: ${charges}`);
            logInfo(`league: ${league}`);
            logInfo(`tasks: ${JSON.stringify(tasks)}`);
            logInfo(`boosts: ${JSON.stringify(boosts)}`);

            // Xử lý repaint/start với số lượng charges
            await makePostRequestWithCharges(tgWebAppData, charges);

            // Kiểm tra và thực hiện các request dựa trên tasks
            const taskChecks = [
                { task: "x:notcoin", url: 'https://notpx.app/api/v1/mining/task/check/x?name=notcoin' },
                { task: "channel:notcoin", url: 'https://notpx.app/api/v1/mining/task/check/channel?name=notcoin' },
                { task: "x:notpixel", url: 'https://notpx.app/api/v1/mining/task/check/x?name=notpixel' },
                { task: "channel:notpixel_channel", url: 'https://notpx.app/api/v1/mining/task/check/channel?name=notpixel_channel' },
                { task: "leagueBonusSilver", url: 'https://notpx.app/api/v1/mining/task/check/leagueBonusSilver', condition: (league === "silver" || league === "gold" || league === "platinum") },
                { task: "makePixelAvatar", url: 'https://notpx.app/api/v1/mining/task/check/makePixelAvatar' },
                { task: "joinSquad", url: 'https://notpx.app/api/v1/mining/task/check/joinSquad' },
                { task: "paint20pixels", url: 'https://notpx.app/api/v1/mining/task/check/paint20pixels' },
                { task: "invite3frens", url: 'https://notpx.app/api/v1/mining/task/check/invite3frens' },
                { task: "jettonTask", url: 'https://notpx.app/api/v1/mining/task/check/jettonTask' }
            ];

            for (const { task, url, condition = true } of taskChecks) {
                if (!tasks[task] && condition) {
                    await checkAndSendRequest(task, url, tgWebAppData);
                    await delay(delayBetweenRequests);
                }
            }

            // Kiểm tra và nâng cấp các boosts
            await handleBoostUpgrades(userBalance, boosts, tgWebAppData);

            await claimReward(tgWebAppData);
            // Nếu không có lỗi, thoát khỏi vòng lặp
            break;

        } catch (error) {
            const statusCode = error.response ? error.response.status : null;
            if (statusCode === 504 || statusCode === 400) {
                attempts++;
                logInfo(`Thử lại lần ${attempts} sau khi gặp lỗi: ${statusCode}`, 'warning');
                if (attempts < maxAttempts) {
                    await delay(retryDelay); // Chờ 2 giây trước khi thử lại
                } else {
                    logInfo(`Lỗi vẫn tồn tại sau ${maxAttempts} lần thử, dừng lại`, 'error');
                    break; // Thoát sau khi thử lại 3 lần không thành công
                }
            } else {
                logInfo(`Lỗi khác xảy ra: ${error.message}`, 'error');
                break; // Thoát nếu gặp lỗi khác
            }
        }
    }
}


// // Hàm gửi request repaint/start dựa trên số lượng charges, mỗi lần repaint sẽ random 1 cặp giá trị mới (pixelId và newColor)
// async function makePostRequestWithCharges(tgWebAppData, charges) {
//     const session = axios.create({
//         headers: {
//             'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/129.0.0.0 Safari/537.36',
//             'Authorization': `initData ${tgWebAppData}`,
//             'Origin': 'https://app.notpx.app',
//             'Referer': 'https://app.notpx.app/',
//             'accept': 'application/json, text/plain, */*',
//             'cache-control': 'no-cache',
//             'pragma': 'no-cache',
//             'authority': 'notpx.app',
//             'accept-encoding': 'gzip, deflate, br, zstd',
//             'accept-language': 'en-US,en;q=0.9',
//             'sec-ch-ua': '"Microsoft Edge";v="129", "Not=A?Brand";v="8", "Chromium";v="129", "Microsoft Edge WebView2";v="129"',
//             'sec-ch-ua-mobile': '?0',
//             'sec-ch-ua-platform': '"Windows"',
//             'sec-fetch-dest': 'empty',
//             'sec-fetch-mode': 'cors',
//             'sec-fetch-site': 'same-site',
//         }
//     });

//     const pixelData = await getPixelDataFromAPI(); // API trả về danh sách các cặp {pixelId, newColor}

//     if (!pixelData || pixelData.length === 0) {
//         logInfo('Dữ liệu pixelData từ API rỗng hoặc không hợp lệ. Không thể thực hiện repaint.', 'error');
//         return;
//     }

//     for (let i = 0; i < charges; i++) {
//         // Lấy dữ liệu pixel một lần cho mỗi request
//         const randomPixelData = pixelData.length === 1 ? pixelData[0] : pixelData[Math.floor(Math.random() * pixelData.length)];
//         if (!randomPixelData || !randomPixelData.pixelId || !randomPixelData.newColor) {
//             logInfo('Dữ liệu randomPixelData không hợp lệ.', 'error');
//             return;
//         }

//         const { pixelId, newColor } = randomPixelData;

//         let attempt = 0;
//         let maxAttempts = 3; // Tổng cộng 3 lần thử
//         let success = false;

//         while (attempt < maxAttempts && !success) {
//             try {
//                 const response = await session.post('https://notpx.app/api/v1/repaint/start', {
//                     pixelId: pixelId,  // Sử dụng pixelId đã lấy trước đó
//                     newColor: newColor  // Sử dụng newColor đã lấy trước đó
//                 });

//                 logInfo(`Request repaint/start ${i + 1}/${charges}: ${response.status} ${pixelId} ${newColor}`, 'success');
//                 logInfo(`Chi tiết response từ API repaint/start: ${JSON.stringify(response.data)}`, 'info');
                
//                 success = true; // Request thành công, thoát khỏi vòng lặp
//             } catch (error) {
//                 const statusCode = error.response ? error.response.status : null;
//                 attempt++;
//                 if (statusCode === 400 || statusCode === 504) {
//                     logInfo(`Thử lại lần ${attempt}/${maxAttempts} cho request repaint/start do lỗi ${statusCode}`, 'warning');
//                     await delay(2000); // Chờ 2 giây trước khi thử lại
//                 } else {
//                     logInfo(`Lỗi khi gửi request repaint/start ${i + 1}/${charges}: ${error.response ? error.response.status : error.message}`, 'error');
//                     break; // Nếu không phải lỗi 400 hoặc 504 thì thoát vòng lặp
//                 }
//             }
//         }

//         if (!success) {
//             logInfo(`Request repaint/start ${i + 1}/${charges} thất bại sau ${maxAttempts} lần thử.`, 'error');
//         }

//         logInfo(`Chờ trước khi request tiếp theo...`, 'info');
//         await delay(delayBetweenRequests);
//     }
// }

async function makePostRequestWithCharges(tgWebAppData, charges) {
    const session = axios.create({
        headers: {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/129.0.0.0 Safari/537.36',
            'Authorization': `initData ${tgWebAppData}`,
            'Origin': 'https://app.notpx.app',
            'Referer': 'https://app.notpx.app/',
            'accept': 'application/json, text/plain, */*',
            'cache-control': 'no-cache',
            'pragma': 'no-cache',
            'authority': 'notpx.app',
            'accept-encoding': 'gzip, deflate, br, zstd',
            'accept-language': 'en-US,en;q=0.9',
            'sec-ch-ua': '"Microsoft Edge";v="129", "Not=A?Brand";v="8", "Chromium";v="129", "Microsoft Edge WebView2";v="129"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"Windows"',
            'sec-fetch-dest': 'empty',
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'same-site',
        }
    });

    const pixelData = await getPixelDataFromAPI(); // API trả về danh sách các cặp {pixelId, newColor}

    if (!pixelData || pixelData.length === 0) {
        logInfo('Dữ liệu pixelData từ API rỗng hoặc không hợp lệ. Không thể thực hiện repaint.', 'error');
        return;
    }

    const fixedPixel = pixelData[0]; // Giữ cặp cố định đầu tiên
    const availableColors = [
        "#E46E6E", "#FFD635", "#7EED56", "#00CCC0", "#51E9F4", "#94B3FF", "#E4ABFF", "#FF99AA", "#FFB470", "#FFFFFF",
        "#BE0039", "#FF9600", "#00CC78", "#009EAA", "#3690EA", "#6A5CFF", "#B44AC0", "#FF3881", "#9C6926", "#898D90",
        "#6D001A", "#BF4300", "#00A368", "#00756F", "#2450A4", "#493AC1", "#811E9F", "#A00357", "#6D482F", "#000000"
    ];

    // Loại bỏ màu của cặp cố định ra khỏi danh sách màu
    const filteredColors = availableColors.filter(color => color !== fixedPixel.newColor);
    const otherColor = filteredColors[Math.floor(Math.random() * filteredColors.length)];

    // Chạy các request liên tiếp mà không chờ response
    for (let i = 0; i < charges; i++) {
        const isOdd = i % 2 !== 0;

        // Nếu là request lẻ thì dùng màu cố định, nếu chẵn thì dùng màu mới
        const pixelId = fixedPixel.pixelId;
        const newColor = isOdd ? fixedPixel.newColor : otherColor;

        session.post('https://notpx.app/api/v1/repaint/start', {
            pixelId: pixelId,
            newColor: newColor
        })
        .then(response => {
            logInfo(`Request repaint/start ${i + 1}/${charges}: ${response.status} pixelId: ${pixelId}, color: ${newColor}`, 'success');
            logInfo(`Chi tiết response từ API repaint/start: ${JSON.stringify(response.data)}`, 'info');
        })
        .catch(error => {
            logInfo(`Lỗi khi gửi request repaint/start ${i + 1}/${charges}: ${error.response ? error.response.status : error.message}`, 'error');
        });
    }
}




// Hàm kiểm tra và gửi request dựa trên task
async function checkAndSendRequest(taskName, checkUrl, tgWebAppData) {
    const session = axios.create({
        headers: {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/129.0.0.0 Safari/537.36',
            'Authorization': `initData ${tgWebAppData}`,
            'Origin': 'https://app.notpx.app',
            'Referer': 'https://app.notpx.app/',
            'accept': 'application/json, text/plain, */*'
        }
    });

    try {
        logInfo(`Thực hiện request cho task: ${taskName}`, 'info');
        const response = await session.get(checkUrl);
        logInfo(`Response cho task ${taskName}: ${response.status}`, 'success');
        logInfo(`Chi tiết response từ API cho task ${taskName}: ${JSON.stringify(response.data)}`, 'info');
    } catch (error) {
        logInfo(`Lỗi khi thực hiện request cho task ${taskName}: ${error.response ? error.response.status : error.message}`, 'error');
    }
}

// Hàm xử lý nâng cấp các boosts dựa trên userBalance và các chỉ số boosts hiện tại
async function handleBoostUpgrades(userBalance, boosts, tgWebAppData) {
    const boostNames = ["energyLimit", "paintReward", "reChargeSpeed"];
    
    for (const boost of boostNames) {
        const currentLevel = boosts[boost];
        const nextLevel = currentLevel + 1;
        const upgradeCost = boostCosts[nextLevel];

        if (upgradeCost && userBalance >= upgradeCost) {
            logInfo(`Nâng cấp ${boost} từ level ${currentLevel} lên level ${nextLevel} với giá ${upgradeCost} điểm`);
            await upgradeBoost(boost, tgWebAppData);
            await delay(delayBetweenRequests); // Chờ giữa các yêu cầu nâng cấp
        }
    }
}

// Hàm gửi request nâng cấp boost
async function upgradeBoost(boostName, tgWebAppData) {
    const session = axios.create({
        headers: {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/129.0.0.0 Safari/537.36',
            'Authorization': `initData ${tgWebAppData}`,
            'Origin': 'https://app.notpx.app',
            'Referer': 'https://app.notpx.app/',
            'accept': 'application/json, text/plain, */*'
        }
    });

    try {
        const response = await session.get(`https://notpx.app/api/v1/mining/boost/check/${boostName}`);
        logInfo(`Nâng cấp ${boostName} thành công: ${response.status}`, 'success');
        logInfo(`Chi tiết response từ API nâng cấp ${boostName}: ${JSON.stringify(response.data)}`, 'info');
    } catch (error) {
        logInfo(`Lỗi khi nâng cấp ${boostName}: ${error.response ? error.response.status : error.message}`, 'error');
    }
}

// Hàm chính điều khiển quá trình duyệt các folder và thực hiện các bước
async function main() {
    const subfolders = await getSubfolders(baseFolder);

    for (const subfolder of subfolders) {
        const chromePath = path.join(subfolder, 'App', 'Chrome-bin', 'chrome.exe');
        const profilePath = path.join(subfolder, 'Data', 'profile');

        // Kiểm tra xem file chrome.exe và folder profile có thực sự tồn tại không
        const chromeExists = await fs.pathExists(chromePath);
        const profileExists = await fs.pathExists(profilePath);

        if (chromeExists && profileExists) {
            logInfo(`Mở Chrome tại: ${chromePath} với profile: ${profilePath}`, 'info');

            const browser = await openChromeWithProfile(chromePath, profilePath);
            
            try {
                const tgWebAppData = await controlTelegram(browser);
                logInfo(`Lấy được tgWebAppData: ${tgWebAppData}`, 'success');

                // Thực hiện các bước xử lý mining/status, repaint, và kiểm tra tasks
                await handleMiningStatus(tgWebAppData);

            } catch (error) {
                logInfo(`Lỗi khi điều khiển Telegram Web: ${error.message}`, 'error');
            }
            logInfo('Chờ trước khi chuyển sang folder tiếp theo...', 'info');
            await delay(delayBetweenFolder);
            await browser.close();
        } else {
            logInfo(`Không tìm thấy Chrome hoặc profile tại: ${subfolder}`, 'warning');
        }

        // Chờ 7 giây trước khi chuyển sang folder tiếp theo
        
    }
    logInfo('Chờ nghỉ để chạy lần tiếp theo...', 'info');
    await delay(delayBetweenCycles);
}

// Chạy chương trình chính
main().catch(err => logInfo(`Lỗi: ${err.message}`, 'error'));
